#include "TGraphEditor.h"
#include "core/core.h"
#include "tools/ToolManager.h"
#include "widgets/TWMisc.h"
#include "widgets/TWProperties.h"
#include "widgets/TWToolBox.h"
#include "widgets/canvas/TWView.h"
#include "widgets/TCheckableButton.h"
#include "tools/DefaultTool/TDefaultTool.h"
#include "tools/TPosTool.h"
#include "tools/TApplyTool.h"
#include <QTimer>
#include <QSettings>
#include <QApplication>
#include <QLabel>
#include <QComboBox>
#include <QCheckBox>
#include <QProgressBar>

namespace graphed {

TGraphEditor::TGraphEditor(QWidget *parent) : QWidget(parent)
{
    x1 = x2 = y1 = y2 = z1 = z2 = 0;
    xHeadRange = yHeadRange = zHeadRange = xStageRange = yStageRange = zStageRange = 100;

    ToolBox = new TWToolBox(this);
    View = INSTANCE(TWView);
    Misc = new TWMisc(this);
    WProperties = new TWProperties(this);
    //START_FUCK:
    ((QHBoxLayout *)Misc->/*findChild<QComboBox *>()->parentWidget()->*/layout())->insertWidget(0,
                                                                                            INSTANCE(ToolManager)->GetButtonsOfTools().at(8)); //apply button
    //END_FUCK:
    //GOTO START_FUCK
    setWindowTitle(tr("Lithography - NSpec"));

    QGroupBox *cwid = new QGroupBox(this);
    QVBoxLayout *clayout = new QVBoxLayout(cwid);
    clayout->setSpacing(6);
    clayout->setContentsMargins(3, 3, 3, 3);

    QVBoxLayout *LeftLayout = new QVBoxLayout;
    LeftLayout->setAlignment(Qt::AlignTop);
    LeftLayout->setSpacing(6);
    LeftLayout->setContentsMargins(6, 0, 6, 6);

    QGroupBox *Gr_L_LInfo_cg = new QGroupBox(tr("Z Position"), this);
    QVBoxLayout *L_LInfo_cg = new QVBoxLayout(Gr_L_LInfo_cg);

    QHBoxLayout *MainLayout = new QHBoxLayout(this);
    MainLayout->setSpacing(6);
    MainLayout->addWidget(cwid);
    MainLayout->addLayout(LeftLayout);
    MainLayout->addWidget(View, 1);
    MainLayout->addWidget(Gr_L_LInfo_cg);


    QGroupBox *GrSelectBoxs = new QGroupBox(this);
    QVBoxLayout *LSelectBoxs = new QVBoxLayout(GrSelectBoxs);
    LSelectBoxs->setSpacing(6);
    LSelectBoxs->setContentsMargins(6, 6, 6, 6);

    QGroupBox *GrSelectionInfo = new QGroupBox(tr("Selection info").remove(" info"), this);
    QVBoxLayout *LInfo = new QVBoxLayout(GrSelectionInfo);
    LInfo->setSpacing(6);
    LInfo->setContentsMargins(6, 3, 6, 6);

    LeftLayout->addWidget(GrSelectBoxs);
    LeftLayout->addWidget(GrSelectionInfo);
    LeftLayout->addWidget(WProperties);

    CBisContinuous = new QCheckBox(tr("Disable pulse between points"));
    CBisContinuous->setChecked(true);

    BStartStop = new TCheckableButton(makeHugeIcon(":stop.png"), makeHugeIcon(":start.png"), tr("Stop"), tr("Start"));
    connect(BStartStop, SIGNAL(BChecked(bool)), this, SLOT(startPrbar(bool)));

    TxyUpdate = new QTimer(this);
    TxyUpdate->setInterval(100);

    PBProgress = new QProgressBar;
    PBProgress->setRange(0,100);
    PBProgress->setValue(0);

    QHBoxLayout *StartStopLayout = new QHBoxLayout;
    StartStopLayout->setContentsMargins(0, 0, 0, 0);
    StartStopLayout->addWidget(BStartStop);
    StartStopLayout->addWidget(PBProgress, 1);

    QHBoxLayout *DataLayout = new QHBoxLayout;
    DataLayout->setContentsMargins(0, 0, 0, 0);
    DataLayout->addWidget(Misc->findChildren<QPushButton *>().at(0));
    DataLayout->addWidget(Misc->findChildren<QPushButton *>().at(1));
    DataLayout->addWidget(ToolBox->findChildren<QPushButton *>().last());

    View->CanvasController()->SetBackground(QImage(":/Lithography/Bkgr.png"));
    View->SetDocumentRect(QRectF(0, 0, 0.0001, 0.0001));
    View->setContentsMargins(0, 0, 0, 0);
    BLithoType = new QComboBox;
    BLithoType->addItem(makeHugeIcon(":/Lithography/Force.png"), tr("Force"),0);
    BLithoType->addItem(makeHugeIcon(":/Lithography/Voltage.png"), tr("Voltage"),0);
    BLithoType->addItem(makeHugeIcon(":/Lithography/Laser.png"), tr("Laser"),0);
    connect(BLithoType, SIGNAL(currentIndexChanged(int)), this, SLOT(slSetLitoType(int)));


    BScanner = new QComboBox;
    BScanner->addItem(makeHugeIcon(":Head_page.png"),tr("Head"));
    BScanner->addItem(makeHugeIcon(":stage_page.png"),tr("Stage"));
    connect(BScanner, SIGNAL(currentIndexChanged(int)), this, SLOT(sl_setRange(int)));

    LlithoVout = new QLabel(tr("Output channel"));
    LlithoVout->hide();

    BlithoVout = new QComboBox;
    BlithoVout ->addItem(tr("Tip Voltage"));
    BlithoVout ->addItem(tr("DAC 1"));
    BlithoVout ->addItem(tr("DAC 2"));
    BlithoVout ->addItem(tr("DAC AUX 1"));
    BlithoVout ->addItem(tr("DAC AUX 2"));
    BlithoVout->hide();
    connect(BlithoVout, SIGNAL(currentIndexChanged(int)), this, SLOT(slSetVout(int)));


    LInfo_cx = new Edit(0, 200, 3);
    LInfo_cy = new Edit(0, 200, 3);
    LInfo_w = new Edit(1, 200, 3);
    LInfo_h = new Edit(1, 200, 3);

    LInfo_z = new NSTSlider(0, 5, 3, this, true);
    L_LInfo_cg->addWidget(LInfo_z);

    connect(LInfo_z, SIGNAL(sliderReleased()), this, SLOT(setZpos()));
    connect(View, SIGNAL(sig_XYPosition(double,double)), this, SLOT(setXYpos(double,double)));
    connect(LInfo_cx, SIGNAL(newValue()), this, SLOT(SendSelectionParams()));
    connect(LInfo_cy, SIGNAL(newValue()), this, SLOT(SendSelectionParams()));
    connect(LInfo_w, SIGNAL(newValue()), this, SLOT(SendSelectionParams()));
    connect(LInfo_h, SIGNAL(newValue()), this, SLOT(SendSelectionParams()));

    QHBoxLayout *LInfo_cg = new QHBoxLayout();
    QHBoxLayout *LInfo_sg = new QHBoxLayout();
    LInfo_cg->setSpacing(6);
    LInfo_sg->setSpacing(6);
    LInfo_cg->setContentsMargins(0, 0, 0, 0);
    LInfo_sg->setContentsMargins(0, 0, 0, 0);

    LInfo->addLayout(LInfo_cg);
    LInfo->addLayout(LInfo_sg);

    QGroupBox *LInfo_01 = new QGroupBox(tr(" x position "));
    QGroupBox *LInfo_02 = new QGroupBox(tr(" y position "));
    QGroupBox *LInfo_03 = new QGroupBox(tr(" width "));
    QGroupBox *LInfo_04 = new QGroupBox(tr(" height "));
    QHBoxLayout *LInfo_1 = new QHBoxLayout(LInfo_01);
    QHBoxLayout *LInfo_2 = new QHBoxLayout(LInfo_02);
    QHBoxLayout *LInfo_3 = new QHBoxLayout(LInfo_03);
    QHBoxLayout *LInfo_4 = new QHBoxLayout(LInfo_04);
    LInfo_1->setContentsMargins(0, 3, 0, 0);
    LInfo_2->setContentsMargins(0, 3, 0, 0);
    LInfo_3->setContentsMargins(0, 3, 0, 0);
    LInfo_4->setContentsMargins(0, 3, 0, 0);
    LInfo_01->setFlat(true);
    LInfo_02->setFlat(true);
    LInfo_03->setFlat(true);
    LInfo_04->setFlat(true);
    LInfo_01->setAlignment(Qt::AlignHCenter);
    LInfo_02->setAlignment(Qt::AlignHCenter);
    LInfo_03->setAlignment(Qt::AlignHCenter);
    LInfo_04->setAlignment(Qt::AlignHCenter);

    LInfo_cg->addWidget(LInfo_01);
    LInfo_cg->addWidget(LInfo_02);
    LInfo_sg->addWidget(LInfo_03);
    LInfo_sg->addWidget(LInfo_04);

    LInfo_1->addWidget(LInfo_cx);
    LInfo_1->addWidget(new QLabel(tr("mkm")));
    LInfo_2->addWidget(LInfo_cy);
    LInfo_2->addWidget(new QLabel(tr("mkm")));
    LInfo_3->addWidget(LInfo_w);
    LInfo_3->addWidget(new QLabel(tr("mkm")));
    LInfo_4->addWidget(LInfo_h);
    LInfo_4->addWidget(new QLabel(tr("mkm")));


    QHBoxLayout *LaylithoVout = new QHBoxLayout;
    LaylithoVout->addWidget(LlithoVout);
    LaylithoVout->addWidget(BlithoVout);

    QHBoxLayout *LScannerLayout = new QHBoxLayout;
    LScannerLayout->addWidget(new QLabel(tr("Scanner")));
    LScannerLayout->addWidget(BScanner);

    QHBoxLayout *LTypeLayout = new QHBoxLayout;
    LTypeLayout->addWidget(new QLabel(tr("Lithography Type")));
    LTypeLayout->addWidget(BLithoType);

    LSelectBoxs->addLayout(LScannerLayout);
    LSelectBoxs->addLayout(LTypeLayout);
    LSelectBoxs->addLayout(LaylithoVout);
    LSelectBoxs->addWidget(CBisContinuous);
    LSelectBoxs->addWidget(Misc);
    LSelectBoxs->addLayout(StartStopLayout);
    LSelectBoxs->addLayout(DataLayout);

    clayout->addWidget(ToolBox);

    connect(INSTANCE(ToolManager), SIGNAL(SetLockRequest(bool)), this, SLOT(SetLocked(bool)));
    connect(INSTANCE(ToolManager), SIGNAL(Params(qreal,qreal,qreal,qreal)), this, SLOT(SetParams(qreal,qreal,qreal,qreal)));
}

void TGraphEditor::SendSelectionParams()
{
    SetSelectionParams(LInfo_cx->value(), LInfo_cy->value(),
                       LInfo_w->value(), LInfo_h->value());
}

void TGraphEditor::SetParams(qreal cx, qreal cy, qreal w, qreal h)
{
    LInfo_cx->setValue(cx);
    LInfo_cy->setValue(cy);
    LInfo_w->setValue(w);
    LInfo_h->setValue(h);
}

void TGraphEditor::startPrbar(bool st)
{
    if (st) {
        PBProgress->setRange(0, 0);
        PBProgress->setValue(0);
        TxyUpdate->start();

        INSTANCE(ToolManager)->GetButtonsOfTools().at(7)->click(); //activate postool

        TApplyTool *ApplyTool = (TApplyTool *)FindTool("TApplyTool");
        if (!ApplyTool->ResultsList().size())
            ApplyTool->Activate();

    } else {
        PBProgress->setRange(0, 1);
        PBProgress->setValue(0);
        TxyUpdate->stop();

        if (BScanner->currentIndex() == 0) {
            emit syncX1pos(x1);
            emit syncY1pos(y1);
        } else {
            emit syncX2pos(x2);
            emit syncY2pos(y2);
        }
    }
}

void TGraphEditor::sl_setHeadRange(double xr,double yr, double zr)
{
    xHeadRange = xr;
    yHeadRange = yr;
    zHeadRange = zr;

    syncXYZrange();
}
void TGraphEditor::sl_setStageRange(double xr,double yr, double zr)
{
    xStageRange = xr;
    yStageRange = yr;
    zStageRange = zr;

    syncXYZrange();
}
void TGraphEditor::updZ1(double z) {
    z1 = z;
    syncZ();
}
void TGraphEditor::updZ2(double z) {
    z2 = z;
    syncZ();
}
void TGraphEditor::setX1(double x)
{
    x1 = x;
    syncXY();
}
void TGraphEditor::setX2(double x)
{
    x2 = x;
    syncXY();
}
void TGraphEditor::setY1(double y)
{
    y1 = y;
    syncXY();
}
void TGraphEditor::setY2(double y)
{
    y2 = y;
    syncXY();
}
void TGraphEditor::syncXY()
{
    QPointF pos;
    if (BScanner->currentIndex() == 0)
        pos = QPointF(x1, y1);
    else
        pos = QPointF(x2, y2);

    graphed::TPosTool *PosTool = (graphed::TPosTool *)FindTool("TPosTool");
    if (PosTool) PosTool->SetPos(pos * 1e-6);
}
void TGraphEditor::syncZ()
{
    LInfo_z->setValue((BScanner->currentIndex() == 0) ? z1 : z2);
}
void TGraphEditor::syncXYZrange()
{
    double xr, yr, zr;
    if (BScanner->currentIndex() == 0) {
        xr = xHeadRange;
        yr = yHeadRange;
        zr = zHeadRange;
    } else {
        xr = xStageRange;
        yr = yStageRange;
        zr = zStageRange;
    }

    View->SetDocumentRect(QRectF(0, 0, (1e-6)*xr, (1e-6)*yr));
    LInfo_z->setMainRangeAndStep(0, zr, 0.001);
}

void TGraphEditor::sl_setRange(int type)
{
    QSettings settings("Param/lithography.conf", QSettings::IniFormat);
    settings.setValue("ScannerType", type);

    syncXYZrange();
    syncXY();
    syncZ();
}
void TGraphEditor::setZpos() {
    double zpos = LInfo_z->value;
    if (BScanner->currentIndex() == 0)
        emit sig_ZPosition(zpos);
    else
        emit sig_ZPosition2(zpos);
}
void TGraphEditor::setXYpos(double x, double y) {
    double xpos = 1e6*x, ypos = 1e6*y;
    if (BScanner->currentIndex() == 0)
        emit sig_XYPosition(xpos, ypos);
    else
        emit sig_XYPosition2(xpos, ypos);
}

void TGraphEditor::SetLocked(const bool Lock)
{
    ToolBox->SetLocked(Lock);
    View->SetLocked(Lock);
    Misc->SetLocked(Lock);
}
void TGraphEditor::updatePrbar()
{
    PBProgress->setValue(PBProgress->value()+1);
}

void TGraphEditor::Finish()
{
    QSettings settings("Param/gui.conf", QSettings::IniFormat);
    if (settings.value("sound_notification", true).toBool())
        qApp->beep();
    BStartStop->setChecked(false); //emits stop signal too
    PBProgress->setValue(PBProgress->maximum());

    if (INSTANCE(ToolManager)->GetToolProxy()->ActiveTool == FindTool("TPosTool"))
        INSTANCE(ToolManager)->GetButtonsOfTools().at(0)->click(); //activate interacttool
}
void TGraphEditor::slSetVout(int type)
{
    QSettings settings("Param/lithography.conf", QSettings::IniFormat);
    settings.setValue("VoutType", type);
}
void TGraphEditor::slSetLitoType(int type)
{
    QSettings settings("Param/lithography.conf", QSettings::IniFormat);
    settings.setValue("LithoType", type);

    switch(type)
    {
    case 0: WProperties->setForceLithName(); LlithoVout->hide(); BlithoVout->hide(); break;
    case 1: WProperties->setVoltLithName(); LlithoVout->show(); BlithoVout->show(); break;
    case 2: WProperties->setLaserMode(); LlithoVout->hide(); BlithoVout->hide(); break;
    }

}
void TGraphEditor::closeEvent(QCloseEvent *event)
{

}

}
