#ifndef PTRCONTAINERS_H
#define PTRCONTAINERS_H

#include <vector>
#include <utility>
#include <algorithm>
#include <QList>

namespace graphed {

template <class TObj, class TGenPtrCont>
class TPushBack;

template <class T>
class TGeneralPtrCont
   {
      template <class TObj, class TGenPtrCont>
      friend class TPushBack;

      template <class X>
      X* CreateNew(X*) { return new X; }

   public:
      T Cont;
      typedef typename T::size_type size_type;
      typedef typename T::value_type value_type;
      typedef typename T::iterator iterator;
      typedef typename T::const_iterator const_iterator;
      typedef typename T::reference reference;
      typedef typename T::const_reference const_reference;
      iterator erase(iterator Position) { return Cont.erase(Position); }
      iterator erase(iterator First, iterator Last ) { return Cont.erase(First, Last); }
      iterator insert(iterator Position, const value_type &X) { return Cont.insert(Position, X); }
      void insert(iterator Position, size_type N, const value_type &X) { Cont.insert(Position, N, X); }
      template <class InputIterator>
      void insert(iterator Position, InputIterator First, InputIterator Last) { Cont.insert(Position, First, Last); }
      size_type size() const        { return Cont.size(); }
      iterator begin()              { return Cont.begin(); }
      const_iterator begin() const  { return Cont.begin(); }
      iterator end()                { return Cont.end(); }
      const_iterator end() const    { return Cont.end(); }
      reference back()              { return Cont.back(); }
      const_reference back() const  { return Cont.back(); }
      reference front()             { return Cont.front(); }
      const_reference front() const { return Cont.front(); }
      virtual void resize(size_type Num)
         {
            if (Cont.size() > Num)      for (; Cont.size() > Num; delete Cont.back(), Cont.pop_back());
            else if (Cont.size() < Num) for (; Cont.size() < Num; Cont.push_back(CreateNew(value_type())));
         }
      virtual void DeleteAll()                   { resize(0); }
      virtual void clear()                       { DeleteAll(); }
      virtual void pop_back()                    { if (!size()) { delete back(); Cont.pop_back(); } }

      TGeneralPtrCont() { }
      TGeneralPtrCont(const size_type Num) { resize(Num); }
      virtual ~TGeneralPtrCont() { DeleteAll(); }

   protected:
      virtual value_type push_back()             { resize(size() + 1); return back(); }
      virtual value_type push_back(const value_type &Ptr) { Cont.push_back(Ptr); return back(); }
   };


template <class TObj, class TGenPtrCont>
class TPushBack
   {
      typedef TPushBack<TObj, TGenPtrCont> TSelf;
   protected:
      TGenPtrCont *GenPtrCont;
   public:
      TPushBack() { }
      virtual TObj *push_back(const TObj &Obj) { GenPtrCont->Cont.push_back(&(*(new TObj) = Obj)); return GenPtrCont->back(); }
      virtual TObj *push_back() { return GenPtrCont->push_back(); }
      virtual TObj *push_back(TObj * const &Ptr) { return GenPtrCont->push_back(Ptr); }
      virtual TSelf &operator<<(const TObj &Obj) { push_back(Obj); return *this; }
      virtual TSelf &operator<<(TObj * const &Ptr) { push_back(Ptr); return *this; }
   };

template <class T, template <class> class TCont>
class TPtrCont1: public TGeneralPtrCont<TCont<T *> >, public TPushBack<T, TGeneralPtrCont<TCont<T *> > >
   {
      typedef TGeneralPtrCont<TCont<T *> > GenBaseClass;
      typedef TPushBack<T, GenBaseClass> PBBaseClass;
      void Construct() { PBBaseClass::GenPtrCont = this; }
   protected:
      TPtrCont1(): TGeneralPtrCont<TCont<T *> >() { Construct(); }
      TPtrCont1(const typename TGeneralPtrCont<TCont<T *> >::size_type Num): TGeneralPtrCont<TCont<T *> >(Num)
         { Construct(); }
      virtual ~TPtrCont1() { }
   };

template <class T>
class TPtrQList: public TPtrCont1<T, QList>
   {
   public:
      typedef TPtrCont1<T, QList> BaseClass;
      TPtrQList(): TPtrCont1<T, QList>() { }
      TPtrQList(const typename BaseClass::size_type Num): TPtrCont1<T, QList>(Num) { }
      ~TPtrQList()                                  { }
      T &operator[](typename BaseClass::size_type Num)                                 { return *BaseClass::Cont[Num]; }
      const T &operator[](typename BaseClass::size_type Num) const                     { return *BaseClass::Cont[Num]; }
      T *&operator()(typename BaseClass::size_type Num)               { return BaseClass::Cont[_CheckIndex(Num, BaseClass::Cont, "T *TPtrQList::operator()(size_type Num) ")]; }
      T * const &operator()(typename BaseClass::size_type Num) const  { return BaseClass::Cont[_CheckIndex(Num, BaseClass::Cont, "T *TPtrQList::operator()(size_type Num) const ")]; }
      void Delete(typename BaseClass::size_type Num)
         {
            _CheckIndex(Num, BaseClass::Cont, "TPtrQList::Delete(size_type Num)");
            delete BaseClass::Cont[Num]; BaseClass::Cont.erase(BaseClass::Cont.begin() + Num);
         }
   };

}

#endif // PTRCONTAINERS_H
