#ifndef TFACTORY_H
#define TFACTORY_H

#include <QSharedPointer>
#include <QString>
#include <QMap>

namespace graphed {

template <class Base>
class TFactory
   {
   public:
      typedef QSharedPointer<Base> BaseSP;
      template <class Derived>
      void Register(const QString &Name) { Factories[Name] = BaseTypeSP(new DerivedType<Derived>); }
      BaseSP Create(const QString &Name)
         {
            if (Factories.find(Name) == Factories.end()) return BaseSP();
            return Factories[Name]->Create();
         }

   private:
      class BaseType
         {
         public:
            virtual ~BaseType() {}
            virtual BaseSP Create() const = 0;
         };
      typedef QSharedPointer<BaseType> BaseTypeSP;

      template <class T>
      class DerivedType: public BaseType
         {
         public:
            virtual BaseSP Create() const { return BaseSP(new T); }
         };

      typedef QMap<QString, BaseTypeSP> TFactoryContainer;
      TFactoryContainer Factories;
   };

}

#endif // TFACTORY_H
