#ifndef GRAPHED_CORE_H
#define GRAPHED_CORE_H

#include <QRectF>
#include <QList>
#include <QIcon>
#include <QString>

QIcon makeHugeIcon(QString path); //controls.h

namespace graphed {

#define INSTANCE(__SINGL) __SINGL::Instance(__LINE__, __FILE__)

#define QTPIMPL_PRIVATE(__class) \
   Q_DECLARE_PRIVATE(__class);  \
   protected:                   \
      __class##Private  * const d_ptr; \
   private:

#define QTPIMPL_PUBLIC(__class) \
   Q_DECLARE_PUBLIC(__class); \
   public:                    \
   __class * q_ptr;



template <class T>
class TSettableVar
   {
   public:
      T Var;
      bool ValueIsSet;
      TSettableVar()                                          { ValueIsSet = false; }
      TSettableVar(const T &Src)                              { ValueIsSet = true; Var = Src; }
      T &operator=(const T &Src)                              { ValueIsSet = true; if (&Var == &Src) return Var; return Var = Src; }
      TSettableVar<T> &operator=(const TSettableVar<T> &Src)  { if (this == &Src); else if ((ValueIsSet = Src.IsValueSet())) Var = Src.Var; return *this; }
      TSettableVar<T> &operator<=(const TSettableVar<T> &Src) { if (Src.IsValueSet()) *this = Src; return *this; } //perform copy ONLY IF src.ValueIsSet == true!!!
      bool operator==(const T &Src) const                     { return Var == Src; }
      bool operator==(const TSettableVar<T> &Src) const       { return Var == Src.Var; }
      bool operator!=(const T &Src) const                     { return Var != Src; }
      bool operator!=(const TSettableVar<T> &Src) const       { return Var != Src.Var; }
      operator T() const                                      { return Var; }
      operator T()                                            { return Var; }
      T operator+(const T &Src) const                         { return Var + Src; }
      T operator-(const T &Src) const                         { return Var - Src; }
      T operator*(const T &Src) const                         { return Var * Src; }
      T operator/(const T &Src) const                         { return Var / Src; }
      T &operator+=(const T &Src)                             { return Var += Src; }
      T &operator-=(const T &Src)                             { return Var -= Src; }
      T &operator*=(const T &Src)                             { return Var *= Src; }
      T &operator/=(const T &Src)                             { return Var /= Src; }
      T &operator|=(const T &Src)                             { return Var |= Src; }
      bool IsValueSet() const                                 { return ValueIsSet; }
   };

typedef TSettableVar<QRectF> TSettableVar_RF;
typedef TSettableVar<QPointF> TSettableVar_PF;
typedef QList<QPointF> TPointFList;

typedef QVector<qreal> STLRealVector;
typedef std::vector<QPoint> STLPointVector;
typedef QVector<QPointF> STLPointFVector;

class GraphEd
   {
      static bool CompareY(const QPointF &P1, const QPointF &P2) { return P1.y() < P2.y(); }

   public:
      enum THandles {TopLeftHandle = 0,
                     TopMiddleHandle,
                     TopRightHandle,
                     RightMiddleHandle,
                     BottomRightHandle,
                     BottomMiddleHandle,
                     BottomLeftHandle,
                     LeftMiddleHandle,
                     NoHandle };

      enum TShapeSelection
         {
            Selected,   ///< return the first selected with the highest z-ordering (i.e. on top).
            Unselected, ///< return the first unselected on top.
            //NextUnselected, ///< return the first unselected directly under a selected shape, or the top most one if nothing is selected.
            ShapeOnTop  ///< return the shape highest z-ordering, regardless of selection.
         };

      enum TKnownZIndexes
         {
            BackgroundIndex = -2,
            ResultIndex = -1
         };

      static QString ResultShapeName() { return "ResultShape"; }
      static QString BackgroundShapeName() { return "BackgroundShape"; }


      static TPointFList GetHandlePoints(const QRectF &Rect, const bool DoSort = true);
      static TPointFList GetHandlePoints(const QPointF &TL, const QPointF &TR, const QPointF &BR, const QPointF &BL, const bool DoSort = true);
      static TPointFList GetHandlePoints(const TPointFList &Corners, const bool DoSort = true);
      template <class T>
      static T Sign(const T &t) { return t >= 0 ? static_cast<T>(1) : static_cast<T>(-1); }
   };

}

#endif // GRAPHED_CORE_H
