#ifndef TRESULTS_H
#define TRESULTS_H
#include "core/core.h"
#include "core/PtrContainers.h"

#include <QPair>

namespace graphed {
namespace out {

class TResults;
typedef TPtrQList<TResults> TResultsList;
typedef QPair<qreal, qreal> TVtData;

class TResults
{
protected:
    const STLPointFVector m_PointList;
    const qreal m_t1;
    const qreal m_Speed;

public:
    TResults(): m_t1(0.), m_Speed(0)  { }
    TResults(const STLPointFVector &L, const qreal t1, const qreal Speed) :
        m_PointList(L), m_t1(t1), m_Speed(Speed) { }
    virtual ~TResults() { }
    const STLPointFVector &PointList() const { return m_PointList; }
    virtual TVtData VtData1(STLPointFVector::const_iterator) const { return TVtData(0, m_t1); }
    TResults &operator = (const TResults &) { return *this; }
    virtual qreal Speed() const { return m_Speed; }

};

class TConstVoltageRes: public TResults
{
    const qreal V1;
public:
    TConstVoltageRes(const STLPointFVector &L, const TVtData &Vt1, const qreal Speed):
        TResults(L, Vt1.second, Speed), V1(Vt1.first) { }
    virtual TVtData VtData1(STLPointFVector::const_iterator) const { return TVtData(V1, m_t1); }
};

class TVarVoltage: public TResults
{
    const STLRealVector m_VList;
    qreal FindVoltage(STLPointFVector::const_iterator PointIt) const;
    const qreal m_MaxV;
public:
    TVarVoltage(const STLPointFVector &L, const qreal t1, const STLRealVector &VList, const qreal Speed, const qreal MaxV):
        TResults(L, t1, Speed), m_VList(VList), m_MaxV(MaxV) { }
    virtual TVtData VtData1(STLPointFVector::const_iterator PointIt) const;
    qreal MaxV() const { return m_MaxV; }
};

STLPointVector Line(QPoint p1, QPoint p2);
STLPointVector Circle(const QPoint &c, const int r);

}

}

#endif // TRESULTS_H
