#ifndef SHAPEMANAGER_H
#define SHAPEMANAGER_H

#include "shape/TShape.h"
#include "shape/TSelectionShape.h"
#include "core/core.h"

namespace graphed {

class ShapeManagerPrivate;
class ShapeManager: public QObject
   {
      Q_OBJECT
      QTPIMPL_PRIVATE(ShapeManager)

   signals:
      void ShapeChanged(TShapeSP Shape);

   private:

      ShapeManager();
      ~ShapeManager();
      static ShapeManager *Self;
      static bool IAmBeingCreatedNow;
   public:
      static ShapeManager *Instance(const int Line = 0, const char *File = NULL);
      static void Destroy();

      //void update();
      TShapeList FindAll(const QString &ObjName) const;
      void AddShape(TShapeSP s);
      void RemoveAllExcept(const QString &Name);
      void RemoveAllExcept(const QStringList &Names);
      void RemoveAll(const QString &ObjName);
      void Remove(const TShapeList &Cont);
      template <class T>
      void Remove(const T &Cont)
         {
            TShapeList List;
            for (typename T::const_iterator It = Cont.begin(); It != Cont.end;  List << *(It++));
            Remove(List);
         }
      TShapeList::size_type count() const;
      const TShapeList &ShapeList() const;
      TShapeList &ShapeList();


      TShapeSP ShapeAt(const QPointF &p, GraphEd::TShapeSelection SelMode) const;
      void paint(QPainter *p, const TConversionBase *Convert) const;

      const TSelectionShapeSP Selection() const;
      TSelectionShapeSP Selection();

      QRectF BoundingRect() const;
      void NotifyShapeChanged(TShape *Sh);

      int MaxZIndex() const;
   };

}

#endif // SHAPEMANAGER_H
