#include "TEllipseCurveShape.h"
#include "TEllipseCurveShape_p.h"
#include "widgets/canvas/TWView.h"
#include "widgets/canvas/TConversion.h"
#define _USE_MATH_DEFINES
#include <math.h>
#include <QVector2D>
#include <QDebug>

namespace graphed {

TSettableVar_RF TEllipseCurveShapePrivate::UpdateBoundingRect() const
{
    BoundingRect = QRectF();
    if (!IsPointsValid()) return BoundingRect;
    UpdateR();
    return BoundingRect = QRectF(Center - QPointF(R, R), Center + QPointF(R, R));
}
bool TEllipseCurveShapePrivate::IsPointsValid() const
{ return Center.IsValueSet() && FreePoint.IsValueSet(); }
void TEllipseCurveShapePrivate::UpdateR() const
{
    if (!IsPointsValid()) return;
    R = sqrt(pow(Center.Var.x() - FreePoint.Var.x(), 2.) + pow(Center.Var.y() - FreePoint.Var.y(), 2.));
}

//======================================================================================================================

TEllipseCurveShape::TEllipseCurveShape(): TDataShape<out::TVtData>(*new TEllipseCurveShapePrivate)
{
    setObjectName(ShapeName());
    SetAngles(QPointF(0, 270));
    Q_D(TEllipseCurveShape);
    d->State = 0;
    d->Transformation = QTransform();
    d->Inverse_flag = 1;

    d->__x_set = false;
    d->__y_set = false;
    d->__r_set = false;
    d->__ratio_not_radius_set = false;
    d->_rotation_angle = 0;
    d->_y_scale = 1;
}

void TEllipseCurveShape::SetNextParam() { Q_D(TEllipseCurveShape); d->State++; };
void TEllipseCurveShape::SetFloatParameter(int id, qreal val)
{
    Q_D(TEllipseCurveShape);

    if (id == 10) {
        d->Center.ValueIsSet = true;
        d->FreePoint.ValueIsSet = true;
        d->Center.Var.setX(val);
        d->FreePoint.Var.rx() += val;
    } else if (id == 20) {
        d->Center.ValueIsSet = true;
        d->FreePoint.ValueIsSet = true;
        d->Center.Var.setY(val);
        d->FreePoint.Var.ry() += val;
    } else if (id == 40) {
        d->__r_set = true;
        d->R = val;
        d->FreePoint.ValueIsSet = true;
        d->FreePoint.Var.rx() += d->R;
    } else if (id == 41) {
        d->Angles.ValueIsSet = true;
        d->Angles.Var.rx() = val*180.0/M_PI + 180;
    } else if (id == 42) {
        d->Angles.ValueIsSet = true;
        d->Angles.Var.ry() = val*180.0/M_PI - (d->Angles.Var.x() - 180);
        if (val > d->Angles.Var.x() - 180)
            d->Angles.Var.ry() += 360;
    } else if (id == 50) {
        d->Angles.ValueIsSet = true;
        d->Angles.Var.rx() = val;
    } else if (id == 51) {
        d->Angles.ValueIsSet = true;
        d->Angles.Var.ry() = val - d->Angles.Var.x();
        if (val < d->Angles.Var.x())
            d->Angles.Var.ry() += 360;
    }

    if (id == 11) {
        d->__x_val = val;
        d->__x_set = true;
    } else if (id == 21) {
        d->__y_val = val;
        d->__y_set = true;
    }

    if (d->__x_set && d->__y_set) {
        d->__x_set = false;
        d->__y_set = false;
        d->__ratio_not_radius_set = true;

        qreal angle1 = 180.0/M_PI*asin( d->__y_val/sqrt(d->__x_val*d->__x_val + d->__y_val*d->__y_val) );
        if (d->__x_val > 0) angle1 = 180 - angle1;
        d->_rotation_angle = -angle1;
    }

    if (d->__ratio_not_radius_set && d->__r_set) {
        d->__ratio_not_radius_set = false;
        d->__r_set = false;

        d->_y_scale = d->R;

        d->FreePoint.Var.rx() -= d->R;
        d->R = sqrt(d->__x_val*d->__x_val + d->__y_val*d->__y_val);
        d->FreePoint.Var.rx() += d->R;
    }

    d->UpdateBoundingRect();
}

void TEllipseCurveShape::SetAngles(const QPointF &p)
{
    Q_D(TEllipseCurveShape);
    d->Angles = p;
    d->UpdateBoundingRect();
}
void TEllipseCurveShape::SetCenter(const QPointF &p)
{
    Q_D(TEllipseCurveShape);
    d->Center = p;
    d->UpdateBoundingRect();
    SetNextParam();
}

bool TEllipseCurveShape::SetFreePoint(const QPointF &p)
{
    Q_D(TEllipseCurveShape);

    if (d->State == 1) {
        d->FreePoint = p;

        QVector2D dir(d->Center - p);
        qreal angle1 = 180.0/M_PI*asin( dir.y()/dir.length() );
        if (dir.x() > 0) angle1 = 180 - angle1;
        angle1 += 360;
        while (angle1 >= 360)
            angle1 -= 360;

        SetAngles(QPointF(angle1, 270));

        d->UpdateBoundingRect();
        return false;
    } else if (d->State == 2){
        QVector2D dir1(d->Center - p);
        QVector2D dir2(d->Center - d->FreePoint);
        qreal angle0 = d->Angles.Var.x();

        qreal angle1 = 180.0/M_PI*asin( dir1.y()/dir1.length() );
        qreal angle2 = 180.0/M_PI*asin( dir2.y()/dir2.length() );
        if (dir1.x() > 0) angle1 = 180 - angle1;
        if (dir2.x() > 0) angle2 = 180 - angle2;

        angle1 = angle1-angle2;
        angle1 += 360*2;
        while (angle1 >= 360)
            angle1 -= 360;

        SetAngles(QPointF(angle0, angle1));

        d->UpdateBoundingRect();
        return false;
    } else if (d->State == 3){
        QVector2D dir(d->Center - p);
        d->Inverse_flag = dir.y() > 0 ? 1 : -1;

        d->UpdateBoundingRect();
        return true;
    }

    return true;
}
void TEllipseCurveShape::paint(QPainter *p, const TConversionBase * /*Convert*/) const
{
    Q_D(const TEllipseCurveShape);
    if (!d->IsPointsValid()) return;

    p->setPen(d->Pen);

    QPainterPath path;
    path.arcMoveTo(QRectF(-d->R, -d->R, d->R*2, d->R*2), d->Angles.Var.x());
    path.arcTo(QRectF(-d->R, -d->R, d->R*2, d->R*2), d->Angles.Var.x(), d->Angles.Var.y()*d->Inverse_flag);

    QPainterPath scaled = QMatrix().scale(1, d->_y_scale).map(path);
    QPainterPath rotated = QMatrix().rotate(d->_rotation_angle).map(scaled);

    rotated.translate(d->Center.Var);

    p->drawPath(rotated);
}
QRectF TEllipseCurveShape::BoundingRect() const
{
    Q_D(const TEllipseCurveShape);
    if (d->BoundingRect.IsValueSet()) return d->BoundingRect;
    else return d->UpdateBoundingRect();
}
bool TEllipseCurveShape::IsIn(const QPointF &PF) const
{
    Q_D(const TEllipseCurveShape);
    return TShape::IsIn(d->Transformation.inverted().map(PF));
}
out::TResults *TEllipseCurveShape::ToPointList() const
{
    Q_D(const TEllipseCurveShape);
    TConversionBase *Conversion = INSTANCE(TWView)->Conversion();
    STLPointFVector Result;
    STLPointVector ResolutionResult;
    STLPointVector Out;

    QPainterPath path;
    path.arcMoveTo(QRectF(-d->R, -d->R, d->R*2, d->R*2), d->Angles.Var.x());
    path.arcTo(QRectF(-d->R, -d->R, d->R*2, d->R*2), d->Angles.Var.x(), d->Angles.Var.y()*d->Inverse_flag);

    QPainterPath scaled = QMatrix().scale(1, d->_y_scale).map(path);
    QPainterPath rotated = QMatrix().rotate(d->_rotation_angle).map(scaled);

    rotated.translate(d->Center.Var);
    QPolygonF poly= rotated.toSubpathPolygons()[0];
    qDebug() << poly.size();

    foreach (QPointF pnt, poly) {
        Out.push_back(pnt.toPoint());
    }

    if (Out.size() <= 1) return new out::TConstVoltageRes(Result, MetaData1(), Speed());

    for (STLPointVector::const_iterator It = Out.begin(); It != Out.end() - 1; It++)
    {
        const QPoint p1 = d->Transformation.map(*It);
        const QPoint p2 = d->Transformation.map(It[1]);
        const STLPointVector &Line = out::Line(p1, p2);

        for (STLPointVector::const_iterator LineIt = Line.begin(); LineIt != Line.end(); LineIt++)
            ResolutionResult.push_back(Conversion->VirginVRToResolution(*LineIt));
    }

    if (ResolutionResult.size() <= 1) return new out::TConstVoltageRes(Result, MetaData1(), Speed());

    for (STLPointVector::const_iterator It = ResolutionResult.begin(); It != ResolutionResult.end() - 1; It++)
    {
        const STLPointVector &Line = out::Line(*It, It[1]);
        for (STLPointVector::const_iterator LineIt = Line.begin(); LineIt != Line.end(); LineIt++)
            Result.push_back(Conversion->ResolutionToPhysical(*LineIt));
    }

    return new out::TConstVoltageRes(Result, MetaData1(), Speed());
}
void TEllipseCurveShape::SaveToFile(QDataStream &Stream) const
{
    Q_D(const TEllipseCurveShape);
    Stream << ShapeName();
    SaveTransform(Stream);
    Stream << d->Center.Var << d->Angles.Var << d->R << d->Inverse_flag;
}
void TEllipseCurveShape::LoadFromFile(QDataStream &Stream)
{
    Q_D(TEllipseCurveShape);
    LoadTransform(Stream);
    Stream >> d->Center.Var >> d->Angles.Var >> d->R >> d->Inverse_flag;
    d->Center.ValueIsSet = true;
    d->Angles.ValueIsSet = true;
    d->FreePoint = QPointF(d->Center.Var - QPointF(0, d->R));
    d->UpdateBoundingRect();
}

}
