#include "TEllipseShape.h"
#include "TEllipseShape_p.h"
#include "widgets/canvas/TWView.h"
#include "widgets/canvas/TConversion.h"
#include <math.h>

namespace graphed {

TSettableVar_RF TEllipseShapePrivate::UpdateBoundingRect() const
{
    BoundingRect = QRectF();
    if (!IsPointsValid()) return BoundingRect;
    UpdateR();
    return BoundingRect = QRectF(Center - QPointF(R, R), Center + QPointF(R, R));
}
bool TEllipseShapePrivate::IsPointsValid() const
{ return Center.IsValueSet() && FreePoint.IsValueSet(); }
void TEllipseShapePrivate::UpdateR() const
{
    if (!IsPointsValid()) return;
    R = sqrt(pow(Center.Var.x() - FreePoint.Var.x(), 2.) + pow(Center.Var.y() - FreePoint.Var.y(), 2.));
}

//======================================================================================================================

TEllipseShape::TEllipseShape(): TDataShape<out::TVtData>(*new TEllipseShapePrivate)
{ setObjectName(ShapeName()); Q_D(TEllipseShape); d->Transformation = QTransform(); d->FreePoint = QPointF(); }
void TEllipseShape::SetFloatParameter(int id, qreal val)
{
    Q_D(TEllipseShape);

    if (id == 10) {
        d->Center.ValueIsSet = true;
        d->Center.Var.setX(val);
        d->FreePoint.Var.rx() += val;
    } else if (id == 20) {
        d->Center.ValueIsSet = true;
        d->Center.Var.setY(val);
        d->FreePoint.Var.ry() += val;
    } else if (id == 40) {
        d->R = val;
        d->FreePoint.ValueIsSet = true;
        d->FreePoint.Var.rx() += d->R;
    }

    d->UpdateBoundingRect();
}

void TEllipseShape::SetCenter(const QPointF &p)
{
    Q_D(TEllipseShape);
    d->Center = p;
    d->UpdateBoundingRect();
}
void TEllipseShape::SetFreePoint(const QPointF &p)
{
    Q_D(TEllipseShape);
    d->FreePoint= p;
    d->UpdateBoundingRect();
}
void TEllipseShape::paint(QPainter *p, const TConversionBase * /*Convert*/) const
{
    Q_D(const TEllipseShape);
    if (!d->IsPointsValid()) return;
    p->setPen(d->Pen);
    p->drawEllipse(d->Center, d->R, d->R);
}
QRectF TEllipseShape::BoundingRect() const
{
    Q_D(const TEllipseShape);
    if (d->BoundingRect.IsValueSet()) return d->BoundingRect;
    else return d->UpdateBoundingRect();
}
bool TEllipseShape::IsIn(const QPointF &PF) const
{
    Q_D(const TEllipseShape);
    return TShape::IsIn(d->Transformation.inverted().map(PF));
}
out::TResults *TEllipseShape::ToPointList() const
{
    Q_D(const TEllipseShape);
    TConversionBase *Conversion = INSTANCE(TWView)->Conversion();
    STLPointFVector Result;
    STLPointVector ResolutionResult;
    STLPointVector Out = out::Circle(d->Center.Var.toPoint(), d->R);

    if (Out.size() <= 1) return new out::TConstVoltageRes(Result, MetaData1(), Speed());

    Out.push_back(Out.front());
    for (STLPointVector::const_iterator It = Out.begin(); It != Out.end() - 1; It++)
    {
        const QPoint p1 = d->Transformation.map(*It);
        const QPoint p2 = d->Transformation.map(It[1]);
        const STLPointVector &Line = out::Line(p1, p2);

        for (STLPointVector::const_iterator LineIt = Line.begin(); LineIt != Line.end(); LineIt++)
            ResolutionResult.push_back(Conversion->VirginVRToResolution(*LineIt));
    }

    if (ResolutionResult.size() <= 1) return new out::TConstVoltageRes(Result, MetaData1(), Speed());

    ResolutionResult.push_back(ResolutionResult.front());
    for (STLPointVector::const_iterator It = ResolutionResult.begin(); It != ResolutionResult.end() - 1; It++)
    {
        const STLPointVector &Line = out::Line(*It, It[1]);
        for (STLPointVector::const_iterator LineIt = Line.begin(); LineIt != Line.end(); LineIt++)
            Result.push_back(Conversion->ResolutionToPhysical(*LineIt));
    }

    return new out::TConstVoltageRes(Result, MetaData1(), Speed());
}
void TEllipseShape::SaveToFile(QDataStream &Stream) const
{
    Q_D(const TEllipseShape);
    Stream << ShapeName();
    SaveTransform(Stream);
    Stream << d->Center.Var << d->R;
}
void TEllipseShape::LoadFromFile(QDataStream &Stream)
{
    Q_D(TEllipseShape);
    LoadTransform(Stream);
    Stream >> d->Center.Var >> d->R;
    d->Center.ValueIsSet = true;
    d->FreePoint = QPointF(d->Center.Var - QPointF(0, d->R));
    d->UpdateBoundingRect();
}

}
