#include "TPointShape.h"
#include "TPointShape_p.h"
#include "widgets/canvas/TWView.h"
#include "widgets/canvas/TConversion.h"

namespace graphed {

TSettableVar_RF TPointShapePrivate::UpdateBoundingRect() const
{
    BoundingRect = QRectF();
    BoundingRect.Var.setRight(Point.x()-10);
    BoundingRect.Var.setLeft(Point.x()+10);
    BoundingRect.Var.setTop(Point.y()-10);
    BoundingRect.Var.setBottom(Point.y()+10);

    return BoundingRect;
}

TPointShape::TPointShape(): TDataShape<out::TVtData>(*new TPointShapePrivate)
{ setObjectName(ShapeName()); Q_D(TPointShape); d->Transformation = QTransform(); }
void TPointShape::SetFloatParameter(int id, qreal val)
{
    Q_D(TPointShape);

    if (id == 10) {
        d->Point.setX(val);
    } else if (id == 20) {
        d->Point.setY(val);
    }

    d->UpdateBoundingRect();
}

void TPointShape::SetFreePoint(const QPointF &p)
{
    Q_D(TPointShape);
    d->Point = p;
    d->UpdateBoundingRect();
}
void TPointShape::paint(QPainter *p, const TConversionBase * /*Convert*/) const
{
    Q_D(const TPointShape);
    p->setPen(d->Pen);
    p->drawLine(d->Point + QPointF(-10, 0), d->Point + QPointF(10, 0));
    p->drawLine(d->Point + QPointF(0, -10), d->Point + QPointF(0, 10));
}
QRectF TPointShape::BoundingRect() const
{
    Q_D(const TPointShape);
    if (d->BoundingRect.IsValueSet()) return d->BoundingRect;
    else return d->UpdateBoundingRect();
}
bool TPointShape::IsIn(const QPointF &PF) const
{
    Q_D(const TPointShape);
    return TShape::IsIn(d->Transformation.inverted().map(PF));
}
out::TResults *TPointShape::ToPointList() const
{
    Q_D(const TPointShape);
    STLPointFVector Result;
    TConversionBase *Conversion = INSTANCE(TWView)->Conversion();

    Result.push_back(Conversion->ResolutionToPhysical(
                         Conversion->VirginVRToResolution(
                             d->Transformation.map(d->Point.toPoint()))));

    return new out::TConstVoltageRes(Result, MetaData1(), Speed());
}
void TPointShape::SaveToFile(QDataStream &Stream) const
{
    Q_D(const TPointShape);
    Stream << ShapeName();
    SaveTransform(Stream);
    Stream << d->Point;
}
void TPointShape::LoadFromFile(QDataStream &Stream)
{
    Q_D(TPointShape);
    LoadTransform(Stream);
    Stream >> d->Point;
}


}
