#include "TRectShape.h"
#include "TRectShape_p.h"
#include "widgets/canvas/TWView.h"
#include "widgets/canvas/TConversion.h"

namespace graphed {

TSettableVar_RF TRectShapePrivate::UpdateBoundingRect() const
{
    BoundingRect = QRectF();
    if (!IsPointsValid()) return BoundingRect;

    BoundingRect.Var.setRight(StartPoint.Var.x());
    BoundingRect.Var.setLeft(StartPoint.Var.x());
    BoundingRect.Var.setTop(StartPoint.Var.y());
    BoundingRect.Var.setBottom(StartPoint.Var.y());
    UpdateBoundingRect(BoundingRect.Var, EndPoint.Var);
    return BoundingRect;
}
void TRectShapePrivate::UpdateBoundingRect(QRectF &r, const QPointF &PF) const
{
    if (PF.x() < r.left()) r.setLeft(PF.x());
    if (PF.x() > r.right()) r.setRight(PF.x());
    if (PF.y() > r.bottom()) r.setBottom(PF.y());
    if (PF.y() < r.top()) r.setTop(PF.y());
}
bool TRectShapePrivate::IsPointsValid() const
{ return StartPoint.IsValueSet() && EndPoint.IsValueSet(); }

//======================================================================================================================

TRectShape::TRectShape(): TDataShape<out::TVtData>(*new TRectShapePrivate)
{ setObjectName(ShapeName()); }
void TRectShape::SetStartPoint(const QPointF &p)
{
    Q_D(TRectShape);
    d->StartPoint = p;
    d->UpdateBoundingRect();
}
void TRectShape::SetEndPoint(const QPointF &p)
{
    Q_D(TRectShape);
    d->EndPoint = p;
    d->UpdateBoundingRect();
}
void TRectShape::paint(QPainter *p, const TConversionBase * /*Convert*/) const
{
    Q_D(const TRectShape);
    if (!d->IsPointsValid()) return;
    p->setPen(d->Pen);
    p->drawRect(QRectF(d->StartPoint, d->EndPoint));
}
QRectF TRectShape::BoundingRect() const
{
    Q_D(const TRectShape);
    if (d->BoundingRect.IsValueSet()) return d->BoundingRect;
    else return d->UpdateBoundingRect();
}
bool TRectShape::IsIn(const QPointF &PF) const
{
    Q_D(const TRectShape);
    return TShape::IsIn(d->Transformation.inverted().map(PF));
}
out::TResults *TRectShape::ToPointList() const
{
    Q_D(const TRectShape);
    QRectF Rect(d->StartPoint, d->EndPoint);
    TPointFList SrcPoints;
    SrcPoints << Rect.topLeft() << Rect.topRight() << Rect.bottomRight() << Rect.bottomLeft() << Rect.topLeft();
    TConversionBase *Conversion = INSTANCE(TWView)->Conversion();
    STLPointFVector Result;
    for (TPointFList::const_iterator It = SrcPoints.begin(); It != SrcPoints.end() - 1; It++)
    {
        const QPoint p1 = Conversion->VirginVRToResolution(d->Transformation.map(*It));
        const QPoint p2 = Conversion->VirginVRToResolution(d->Transformation.map(It[1]));
        const STLPointVector &Line = out::Line(p1, p2);

        for (STLPointVector::const_iterator LineIt = Line.begin(); LineIt != Line.end(); LineIt++)
            Result.push_back(Conversion->ResolutionToPhysical(*LineIt));
    }
    return new out::TConstVoltageRes(Result, MetaData1(), Speed());
}
void TRectShape::SaveToFile(QDataStream &Stream) const
{
    Q_D(const TRectShape);
    Stream << ShapeName();
    SaveTransform(Stream);
    Stream << d->StartPoint.Var << d->EndPoint.Var;
}
void TRectShape::LoadFromFile(QDataStream &Stream)
{
    Q_D(TRectShape);
    LoadTransform(Stream);
    Stream >> d->StartPoint.Var >> d->EndPoint.Var;
    d->StartPoint.ValueIsSet = d->EndPoint.ValueIsSet = true;
    d->UpdateBoundingRect();
}

}
