#include "TSelectionShape.h"
#include "TSelectionShape_p.h"
#include <cmath>

namespace graphed {

TSelectionShapePrivate::TSelectionShapePrivate()
{
}
TSelectionShapePrivate::~TSelectionShapePrivate()
{
}
TSettableVar_RF TSelectionShapePrivate::UpdateBoundingRect() const
{
    BoundingRect = QRectF();

    foreach (TShapeSP Shape, SelectedShapes)
        if (Shape->Transformation().mapRect(Shape->BoundingRect()).width() >= 1 &&
                Shape->Transformation().mapRect(Shape->BoundingRect()).height() >= 1)
            BoundingRect.Var |= Shape->Transformation().mapRect(Shape->BoundingRect());

    return BoundingRect;
}

//======================================================================================================================

TSelectionShape::TSelectionShape(): TShape(*new TSelectionShapePrivate)   { }
TSelectionShape::~TSelectionShape()   {  }
void TSelectionShape::Select(const TShapeSP Shape)
{
    Q_D(TSelectionShape);

    if (d->SelectedShapes.contains(Shape)) return;
    if (!Shape->Selectable()) return;

    d->SelectedShapes.push_back(Shape);

    if (d->SelectedShapes.size() == 1) SetTransformation(Shape->Transformation());
    else SetTransformation(QTransform());

    emit setenabled(true);

    InvalidateCache();

    emit SelectionChanged();
}
void TSelectionShape::DeSelect(const TShapeSP Shape)
{
    Q_D(TSelectionShape);

    d->SelectedShapes.removeAll(Shape);

    if (!d->SelectedShapes.size()) {
        emit setenabled(false);
        SetTransformation(QTransform());
    }

    InvalidateCache();

    emit SelectionChanged();
}
const TShapeList &TSelectionShape::SelectedShapes() const
{
    Q_D(const TSelectionShape);
    return d->SelectedShapes;
}
TShapeList &TSelectionShape::SelectedShapes()
{
    Q_D(TSelectionShape);
    return d->SelectedShapes;
}
TShapeList::size_type TSelectionShape::count() const
{
    Q_D(const TSelectionShape);
    return d->SelectedShapes.size();
}
void TSelectionShape::DeSelectAll()
{
    Q_D(TSelectionShape);
    d->SelectedShapes.clear();
    SetTransformation(QTransform());
    emit setenabled(false);

    InvalidateCache();

    emit SelectionChanged();
}
QRectF TSelectionShape::BoundingRect() const
{
    Q_D(const TSelectionShape);
    if (d->BoundingRect.IsValueSet()) return d->BoundingRect;
    else return d->UpdateBoundingRect();
}
void TSelectionShape::InvalidateCache()
{
    Q_D(TSelectionShape);
    d->BoundingRect.ValueIsSet = false;
}
bool TSelectionShape::IsSelected(const TShapeSP Shape) const
{
    Q_D(const TSelectionShape);
    return d->SelectedShapes.contains(Shape);
}

}
