#ifndef TSHAPE_H
#define TSHAPE_H
#include "shape/TProgressInfo.h"
#include <QPainter>
#include <QSharedPointer>

namespace graphed {

namespace out { class TResults; }

class TConversionBase;
class TShape;
class TShapePrivate;
typedef QSharedPointer<TShape> TShapeSP;
typedef QList<TShapeSP>           TShapeList;

class TShape: public QObject, public TProgressInfo
   {
      QTPIMPL_PRIVATE(TShape)
   protected:
      TShape(TShapePrivate &dd);
      explicit TShape();

      void SaveTransform(QDataStream &Stream) const;
      void LoadTransform(QDataStream &Stream);

   public:
      virtual ~TShape();
      virtual void update() const;
      virtual void paint(QPainter *p, const TConversionBase *Convert) const = 0;
      static void ApplyConversion(QPainter *p, const TConversionBase *Convert);

      virtual QRectF BoundingRect() const = 0;
      QRectF MappedBoundingRect() const;
      //QPainterPath Outline() const;

      /*void SetSize(const QSizeF &NewSize);
      QSizeF size() const;
      virtual void Rotate(qreal angle);
      virtual qreal Rotation() const;
      QTransform AbsoluteTransformation() const;
      void ApplyAbsoluteTransformation(const QTransform &t);
      void ApplyTransformation(const QTransform &t);
      void setVisible(bool on);
      virtual void SetScale(qreal sx, qreal sy);*/

      virtual void SetFloatParameter(int /*id*/, qreal /*val*/) {}
      virtual void SetIntParameter(int /*id*/, quint32 /*val*/) {}
      virtual void SetPosition(const QPointF &Pos);
      virtual QPointF Position() const;
      virtual bool IsIn(const QPointF &PF) const;

      virtual QTransform Transformation() const;
      virtual void SetTransformation(const QTransform &t);

      static QPointF PosInShapesCoord(const TShapeSP Shape, const QPointF &Point)
         { return Shape->Transformation().inverted().map(Point); }

      static QPointF PosInShapesCoord(const TShape *Shape, const QPointF &Point)
         { return Shape->Transformation().inverted().map(Point); }

      QPointF PosInShapesCoord(const QPointF &Point) const
         { return PosInShapesCoord(this, Point); }

      QPointF GlobalPosition(const QPointF &Pos) const;
      QPointF GlobalPosition() const;

      void NotifyChanged();
      virtual out::TResults *ToPointList() const = 0;

      virtual void SaveToFile(QDataStream &Stream) const = 0;
      virtual void LoadFromFile(QDataStream &Stream) = 0;

      bool Selectable() const;
      void SetSelectable(const bool S);
      bool GeometryProtected() const;
      void SetGeometryProtected(const bool GP);

      static bool CompareShapeByZIndex(TShapeSP S1, TShapeSP S2);
      void SetZIndex(const int Z);
      int ZIndex() const;
   };

}

#endif // TSHAPE_H
