#include "TMoveStrategy.h"

#include "shape/TShape.h"
#include "core/TMouseEvent.h"

#include <QPointF>

namespace graphed {

TMoveStrategy::TMoveStrategy(const TCanvasQP c, TMouseEvent *e): TInteractionStrategy(c, e->posF())
{
    if (!INSTANCE(ShapeManager)->Selection()->count()) return;
    Canvas->setCursor(TMoveStrategy::Cursor());

    if (SelectedShapes.size() > 1) SelectedShapes << INSTANCE(ShapeManager)->Selection();
    foreach (TShapeSP Shape, SelectedShapes) OldPositions << Shape->Position();
}
TMoveStrategy::~TMoveStrategy()
{
    Canvas->unsetCursor();
}

void TMoveStrategy::handleMouseMove(TMouseEvent *e)
{
    if (SelectedShapes.isEmpty()) return;
    QPointF Delta = e->posF() - FirstClick;
    TPointFList::size_type i = 0;
    foreach (TShapeSP Shape, SelectedShapes) Shape->SetPosition(OldPositions[i++] + Delta);
    Canvas->update();
}
void TMoveStrategy::CancelInteraction()
{
    QMouseEvent e(QEvent::None, FirstClick.toPoint(), Qt::NoButton, Qt::MouseButtons(), Qt::KeyboardModifiers());
    TMouseEvent E(&e, FirstClick);
    handleMouseMove(&E);
}

}
