#include "TSelectDecorator.h"
#include "shape/TSelectionShape.h"
#include "core/core.h"
#include "shape/ShapeManager.h"
#include "widgets/canvas/TConversionBase.h"

namespace graphed {

void TSelectDecorator::PaintHandles(QPainter *p, const TShapeSP Shape, const TConversionBase *Convert)
{
    //TShape::ApplyConversion(p, Convert);
    p->setPen(QPen(Qt::black));
    p->setBrush(QBrush(Qt::yellow));
    QRectF Handle(0, 0, 5, 5);
    TPointFList HandlePoints = GraphEd::GetHandlePoints(Shape->BoundingRect());
    foreach (QPointF Point, HandlePoints)
    {                                                                         //VVVVVVVVVVVVVVVVVVV - Because we already took into consideration offset in TCanvas::update()
        Handle.moveCenter(Convert->DocumentToView(Shape->GlobalPosition(Point)) - Convert->Offset());
        p->drawRect(Handle);
    }
}

void TSelectDecorator::paint(QPainter *p, const TConversionBase *Convert)
{
    TShapeList Shapes = INSTANCE(ShapeManager)->Selection()->SelectedShapes();
    if (!Shapes.size()) return;

    foreach (TShapeSP Shape, Shapes)
    {
        p->setPen(QPen(Qt::green));
        p->save();
        TShape::ApplyConversion(p, Convert);
        p->setTransform(Shape->Transformation() * p->transform());
        p->drawRect(Shape->BoundingRect());
        p->restore();
        if (Shapes.size() == 1) PaintHandles(p, Shape, Convert);
    }

    if (Shapes.size() > 1)
    {
        p->setPen(QPen(Qt::blue));
        p->save();
        TShape::ApplyConversion(p, Convert);
        p->setTransform(INSTANCE(ShapeManager)->Selection()->Transformation() * p->transform());
        p->drawRect(INSTANCE(ShapeManager)->Selection()->BoundingRect());
        p->restore();
        PaintHandles(p, INSTANCE(ShapeManager)->Selection(), Convert);
    }
}

}
