#include "TLineTool.h"
#include "shape/ShapeManager.h"

namespace graphed {

void TLineTool::FinishShape()
{
    if (!Shape) return;
    Shape->UnsetFreePoint();
    if (Shape->PointsCount() > 1)
    {
        Shape->SetZIndex(INSTANCE(ShapeManager)->MaxZIndex() + 1);
        INSTANCE(ShapeManager)->AddShape(Shape);
        INSTANCE(ShapeManager)->Selection()->DeSelectAll();
        INSTANCE(ShapeManager)->Selection()->Select(Shape);
    }
    Shape.clear();
    Canvas->UpdateCanvas();
}

TLineTool::TLineTool(TCanvas *c): TTool(c)
{
    Icon = (makeHugeIcon(":/Lithography/line_off.png"));
    ToolTip = (QObject::tr("Draw lines"));
    setObjectName(ToolName());
}
void TLineTool::paint(QPainter *p, const TConversionBase *Convert)
{
    //We will draw here _our_ Shape
    TTool::paint(p, Shape, Convert);
}
void TLineTool::mousePressEvent(TMouseEvent *e)
{
    if (e->buttons() & Qt::LeftButton)
    {
        if (!Shape) Shape = TLinesShapeSP(new TLinesShape);
        Shape->AddPoint(e->posF());
    }
    else if ((e->buttons() & Qt::RightButton) && Shape) FinishShape();
}
void TLineTool::mouseMoveEvent(TMouseEvent *e)
{
    if (!Shape) return;
    Shape->SetFreePoint(e->posF());
    Canvas->UpdateCanvas();
}
void TLineTool::Deactivate()
{
    FinishShape();
}

}
