#include "TPosTool.h"
#include "widgets/canvas/TWView.h"
#include "widgets/canvas/TConversion.h"

namespace graphed {

TPosTool::TPosTool(TCanvas *c): TTool(c)
{
    press = false;
    Icon = (makeHugeIcon(":/Lithography/position.png"));
    ToolTip = (QObject::tr("Set scanner position"));
    setObjectName(ToolName());
}
void TPosTool::paint(QPainter *p, const TConversionBase * /*Convert*/)
{
    p->setPen(QPen(Qt::magenta, 2));
    TConversionBase *Conversion = INSTANCE(TWView)->Conversion();
    p->drawLine(Pos*Conversion->Zoom() + QPointF(-10, 0), Pos*Conversion->Zoom() + QPointF(10, 0));
    p->drawLine(Pos*Conversion->Zoom() + QPointF(0, -10), Pos*Conversion->Zoom() + QPointF(0, 10));
}
void TPosTool::mouseReleaseEvent(TMouseEvent *e)
{
    Pos = e->posF();
    press = false;

    TConversionBase *Conversion = INSTANCE(TWView)->Conversion();
    QPointF point =  Conversion->ResolutionToPhysical(
                Conversion->VirginVRToResolution(Pos));

    INSTANCE(TWView)->sigPos(point.x(), point.y());
    Canvas->UpdateCanvas();
}
void TPosTool::mousePressEvent(TMouseEvent * /*e*/)
{
    press = true;
}
void TPosTool::mouseMoveEvent(TMouseEvent *e)
{
    if (press)
        Pos = e->posF();
    Canvas->UpdateCanvas();
}
void TPosTool::Activate()
{
    Canvas->UpdateCanvas();
}
void TPosTool::Deactivate()
{
    Canvas->UpdateCanvas();
}

void TPosTool::SetPos(QPointF pos)
{
    TConversionBase *Conversion = INSTANCE(TWView)->Conversion();

    Pos = Conversion->PhysicalTo(Conversion->VirginViewRect(), pos);

    Canvas->UpdateCanvas();
}

}
