#ifndef TTOOL_H
#define TTOOL_H

#include <QObject>
#include <QIcon>
#include "core/core.h"
#include "widgets/canvas/TCanvas.h"
#include "core/TMouseEvent.h"
#include "shape/TShape.h"
#include "shape/ShapeManager.h"

namespace graphed {

class TTool: public QObject
{
    friend class TApplyToolPrivate;
    Q_OBJECT
signals:
    void ToolActivated(const QString &Name);
    void SetLockRequest(bool);
private slots:
    virtual void on_ToolButton_clicked(bool Clicked);

public:
    TCanvasQP Canvas;
    QIcon Icon;
    QString ToolTip;

    TTool(TCanvas *c) : Canvas(c) { }
    virtual ~TTool() { }

    virtual void paint(QPainter * /*p*/, const TConversionBase * /*Convert*/) { }
    void paint(QPainter *p, TShapeSP Shape, const TConversionBase *Convert);

    virtual void keyReleaseEvent(QKeyEvent *e) { e->ignore(); }
    virtual void keyPressEvent(QKeyEvent *e) { e->ignore(); }
    virtual void mousePressEvent(TMouseEvent *e) { e->ignore(); }
    virtual void mouseMoveEvent(TMouseEvent *e) { e->ignore(); }
    virtual void mouseReleaseEvent(TMouseEvent *e) { e->ignore(); }
    virtual void mouseDoubleClickEvent(TMouseEvent *e) { e->ignore(); }
    virtual void wheelEvent(TMouseEvent *e) { e->ignore(); }
    virtual void Activate() { }
    virtual void Deactivate() { }
};

typedef QPointer<TTool> TToolQP;

template <class TSomeShape>
void FinishShape(TSomeShape &Shape)
{
    if (Shape->objectName() == GraphEd::BackgroundShapeName())  Shape->SetZIndex(GraphEd::BackgroundIndex);
    else if (Shape->objectName() == GraphEd::ResultShapeName()) Shape->SetZIndex(GraphEd::ResultIndex);
    else Shape->SetZIndex(INSTANCE(ShapeManager)->MaxZIndex() + 1);
    INSTANCE(ShapeManager)->AddShape(Shape);
    if (Shape->objectName() != GraphEd::BackgroundShapeName()) {
        INSTANCE(ShapeManager)->Selection()->DeSelectAll();
        INSTANCE(ShapeManager)->Selection()->Select(Shape);
    }
    Shape.clear();
}

}

#endif // TTOOL_H
