#include "TCheckableButton.h"

namespace graphed {

class TCheckableButtonPrivate
   {
      Q_DECLARE_PUBLIC(TCheckableButton);
   public:
      TCheckableButton * q_ptr;
      TCheckableButtonPrivate();
      virtual ~TCheckableButtonPrivate() { }
      void AdjustButton() ;
      void Init();
      void _q_clicked();

      bool Checked;
      QString CheckedText;
      QString UnCheckedText;
      QIcon CheckedIcon;
      QIcon UnCheckedIcon;
   };

TCheckableButtonPrivate::TCheckableButtonPrivate(): Checked(false)  { }
void TCheckableButtonPrivate::AdjustButton()
{
    Q_Q(TCheckableButton);
    if (Checked)
    {
        q->setText(CheckedText);
        q->setIcon(CheckedIcon);
        q->emit BChecked(true);
    }
    else
    {
        q->setText(UnCheckedText);
        q->setIcon(UnCheckedIcon);
        q->emit BChecked(false);
    }
}
void TCheckableButtonPrivate::_q_clicked()
{
    Checked = !Checked;
    AdjustButton();
}
void TCheckableButtonPrivate::Init()
{
    Q_Q(TCheckableButton);
    QObject::connect(q, SIGNAL(clicked()), q, SLOT(_q_clicked()));
    AdjustButton();
}

TCheckableButton::TCheckableButton(): d_ptr(new TCheckableButtonPrivate)
{
    Q_D(TCheckableButton);
    d->q_ptr = this;
    d->Init();
}
TCheckableButton::TCheckableButton(const QString &CheckedText, const QString &UnCheckedText): d_ptr(new TCheckableButtonPrivate)
{
    Q_D(TCheckableButton);
    d->q_ptr = this;
    d->CheckedText = CheckedText;
    d->UnCheckedText = UnCheckedText;
    d->Init();
}
TCheckableButton::TCheckableButton(const QIcon &CheckedIcon, const QIcon &UnCheckedIcon, const QString &CheckedText, const QString &UnCheckedText): d_ptr(new TCheckableButtonPrivate)
{
    Q_D(TCheckableButton);
    d->q_ptr = this;
    d->CheckedText = CheckedText;
    d->UnCheckedText = UnCheckedText;
    d->CheckedIcon = CheckedIcon;
    d->UnCheckedIcon = UnCheckedIcon;
    d->Init();
}
TCheckableButton::~TCheckableButton() { delete d_ptr; }

bool TCheckableButton::isChecked() const
{
    Q_D(const TCheckableButton);
    return  d->Checked;
}
void TCheckableButton::setChecked(bool Checked)
{
    Q_D(TCheckableButton);
    //emit BChecked(Checked);
    if (d->Checked == Checked) return;
    d->_q_clicked();
}

#include "moc_TCheckableButton.cpp"

}


