#include "TWMisc.h"
#include "files/TYEGFiles.h"
#include "shape/ShapeManager.h"
#include "widgets/canvas/TCanvas.h"
#include "widgets/TCheckableButton.h"
#include "widgets/canvas/TWView.h"

#include <QPushButton>
#include <QHBoxLayout>
#include <QFileDialog>
#include <QMessageBox>
#include <QGroupBox>
#include <QComboBox>
#include <QLabel>

namespace graphed {

class TWMiscPrivate
{
    QTPIMPL_PUBLIC(TWMisc)
    public:
        TWMiscPrivate() {}
    virtual ~TWMiscPrivate() { }
    void CreateGUI();
    void _q_on_XYPoints_activated(const QString &Text);
    QComboBox *CBXPoints;
    QComboBox *CBYPoints;
};


void TWMiscPrivate::CreateGUI()
{
    Q_Q(TWMisc);

    QHBoxLayout *HBLForSaveOpen = new QHBoxLayout(q);
    HBLForSaveOpen->setContentsMargins(0, 0, 0, 0);
    HBLForSaveOpen->setAlignment(Qt::AlignLeft);
    HBLForSaveOpen->setSpacing(10);


    QPushButton *BSave = new QPushButton(makeHugeIcon(":/Lithography/Save.png"), QString());
    QObject::connect(BSave, SIGNAL(clicked()), q, SLOT(on_BSave_clicked()));

    QPushButton *BOpen = new QPushButton(makeHugeIcon(":/Lithography/Open.png"), QString());
    QObject::connect(BOpen, SIGNAL(clicked()), q, SLOT(on_BOpen_clicked()));

    HBLForSaveOpen->addWidget(BOpen);
    HBLForSaveOpen->addWidget(BSave);


    QGroupBox *GrBControl = new QGroupBox;
    QHBoxLayout *HBLForSize = new QHBoxLayout(GrBControl);
    HBLForSize->setContentsMargins(6, 3, 6, 6);
    HBLForSize->setAlignment(Qt::AlignLeft);

    HBLForSaveOpen->addWidget(GrBControl);

    QLabel *LSize = new QLabel(QObject::tr("Points: "));
    QLabel *LSizeX = new QLabel("x");

    CBXPoints = new QComboBox;
    QObject::connect(CBXPoints, SIGNAL(activated(QString)), q, SLOT(_q_on_XYPoints_activated(QString)));
    CBXPoints->addItem("30");  CBXPoints->addItem("60");  CBXPoints->addItem("100"); CBXPoints->addItem("200");
    CBXPoints->addItem("300"); CBXPoints->addItem("400"); CBXPoints->addItem("500"); CBXPoints->addItem("1000");
    CBXPoints->setCurrentIndex(2);

    CBYPoints = new QComboBox;
    QObject::connect(CBYPoints, SIGNAL(activated(QString)), q, SLOT(_q_on_XYPoints_activated(QString)));
    CBYPoints->addItem("30");  CBYPoints->addItem("60");  CBYPoints->addItem("100"); CBYPoints->addItem("200");
    CBYPoints->addItem("300"); CBYPoints->addItem("400"); CBYPoints->addItem("500"); CBYPoints->addItem("1000");
    CBYPoints->setCurrentIndex(2);

    HBLForSize->addWidget(LSize);
    HBLForSize->addWidget(CBXPoints);
    HBLForSize->addWidget(LSizeX);
    HBLForSize->addWidget(CBYPoints);
}

void TWMiscPrivate::_q_on_XYPoints_activated(const QString &Text)
{
    Q_UNUSED(Text);
    QSize Size(CBXPoints->currentText().toDouble(), CBYPoints->currentText().toDouble());
    INSTANCE(TWView)->SetDocumentResolution(Size);
    //INSTANCE(TCanvas)->UpdateCanvas();
}


//======================================================================================================================

TWMisc::TWMisc(QWidget *parent) : QWidget(parent), d_ptr(new TWMiscPrivate)
{
    Q_D(TWMisc);
    d->q_ptr = this;
    d->CreateGUI();
    INSTANCE(TWView)->SetDocumentResolution(QSize(100,100));
    fileName="NST data files/noname.nstlitho";
    //QMetaObject::connectSlotsByName(this);
}
TWMisc::~TWMisc()
{
    delete d_ptr;
}

void TWMisc::on_BSave_clicked()
{
    if (!INSTANCE(ShapeManager)->ShapeList().size())
    {
        QMessageBox::warning(this, tr("Save error"), tr("There is nothing to save."));
        return;
    }
    fileName = QFileDialog::getSaveFileName(this,
                                            tr("Save shapes..."),
                                            fileName,
                                            tr("Lithograthy files (*.nstlitho)"),
                                            0, QFileDialog::DontUseNativeDialog);
    if (fileName.isEmpty()) return;
    if (!fileName.endsWith(".nstlitho")) fileName += ".nstlitho";
    out::TYEGFiles YEGFiles;
    YEGFiles.Save(fileName);
}
void TWMisc::on_BOpen_clicked()
{
    fileName = QFileDialog::getOpenFileName(this,
                                            tr("Load shapes..."),
                                            fileName,
                                            tr("Lithograthy files (*.nstlitho);;Text DXF (*.dxf)"),
                                            0, QFileDialog::DontUseNativeDialog);
    if (fileName.isNull())
        return;

    out::TYEGFiles YEGFiles;
    const TShapeList &Shapes = YEGFiles.Load(fileName);
    INSTANCE(ShapeManager)->ShapeList().append(Shapes);
    INSTANCE(TCanvas)->update();
}
void TWMisc::SetLocked(const bool Lock)
{
    setEnabled(!Lock);
}
}

#include "moc_TWMisc.cpp"
