#include "TWProperties.h"
#include <QGroupBox>
#include <QVBoxLayout>
#include <QLabel>
#include <QObject>
#include <QSettings>
#include "core/core.h"
#include "shape/ShapeManager.h"
#include "shape/TImageShape.h"
#include "shape/TDataShape.h"
#include "out/TConvertor.h"
#include "nstslider.h"
#include "controls.h"
#include "edit.hpp"

namespace graphed {

class TWPropertiesPrivate
{
    Q_DECLARE_PUBLIC(TWProperties);
    void CreateGUI();
public:
    bool gt;
    TWProperties * q_ptr;
    TWPropertiesPrivate(): IgnoreValueChanged(false) { }
    void Construct();
    void GetVtDara(out::TVtData &Vt1);
    void Restore_TVtData_MetaData(const TShapeSP Shape);

    void _q_on_SelectionChanged();
    void _q_on_VtSlider_valueChanged(double );
    void _q_on_LEdSSpeed_valueChanged(double );

    DoubleChanger *DCsV, *DCsT, *DCsS;

    bool IgnoreValueChanged;
    QGroupBox *GrBParameters1;
    QGroupBox *GrBSpeed;

};

void TWPropertiesPrivate::Construct()
{
    Q_Q(TWProperties);

    CreateGUI();

    QObject::connect(INSTANCE(ShapeManager)->Selection().data(), SIGNAL(SelectionChanged()), q, SLOT(_q_on_SelectionChanged()));
    QObject::connect(DCsV, SIGNAL(newValue(double)), q, SLOT(_q_on_VtSlider_valueChanged(double)));
    QObject::connect(DCsT, SIGNAL(newValue(double)), q, SLOT(_q_on_VtSlider_valueChanged(double)));
    QObject::connect(DCsS, SIGNAL(newValue(double)), q, SLOT(_q_on_LEdSSpeed_valueChanged(double)));
    QObject::connect(INSTANCE(ShapeManager)->Selection().data(), SIGNAL(setenabled(bool)), GrBParameters1, SLOT(setEnabled(bool)));
    QObject::connect(INSTANCE(ShapeManager)->Selection().data(), SIGNAL(setenabled(bool)), GrBSpeed, SLOT(setEnabled(bool)));

    GrBParameters1->setEnabled(false);
    GrBSpeed->setEnabled(false);
}
void TWPropertiesPrivate::CreateGUI()
{
    Q_Q(TWProperties);

    GrBParameters1 = new QGroupBox(QObject::tr("Pulse SetPoint Min, pulse time "));
    QVBoxLayout *VBLForParameters1 = new QVBoxLayout(GrBParameters1);
    VBLForParameters1->setContentsMargins(6, 6, 6, 6);
    VBLForParameters1->setSpacing(6);

    GrBSpeed = new QGroupBox(QObject::tr("Speed"));
    QHBoxLayout *VBLSpped = new QHBoxLayout(GrBSpeed);
    VBLSpped->setContentsMargins(6, 6, 6, 6);

    QVBoxLayout *MainLayout = new QVBoxLayout(q);
    MainLayout->setContentsMargins(0, 0, 0, 0);
    MainLayout->setSpacing(3);
    MainLayout->addWidget(GrBParameters1);
    MainLayout->addWidget(GrBSpeed);


    DCsV = new DoubleChanger(out::TVoltColourConvertor::MinVoltage, out::TVoltColourConvertor::MaxVoltage, 3);
    DCsV->layout()->addWidget(new QLabel(QObject::tr("V")));

    DCsT = new DoubleChanger(0, 10, 3);
    DCsT->layout()->addWidget(new QLabel(QObject::tr("s")));

    VBLForParameters1->addWidget(DCsV);
    VBLForParameters1->addWidget(DCsT);

    DCsS = new DoubleChanger(0, 100, 3);
    DCsS->layout()->addWidget(new QLabel(QObject::tr("micro")+QObject::tr("m/s")));

    VBLSpped->addWidget(DCsS);
}

void TWPropertiesPrivate::GetVtDara(out::TVtData &Vt1)
{
    Vt1.first = DCsV->value();
    Vt1.second = DCsT->value();
}
void TWPropertiesPrivate::Restore_TVtData_MetaData(const TShapeSP Shape)
{
    if (TMetaData<out::TVtData> *Data = dynamic_cast<TMetaData<out::TVtData> *>(Shape.data()))
    {
        bool OldIgnoreValueChanged = IgnoreValueChanged;
        IgnoreValueChanged = true;

        DCsV->setValue(Data->MetaData1().first);
        DCsT->setValue(Data->MetaData1().second);
        DCsS->setValue(Data->Speed());

        IgnoreValueChanged = OldIgnoreValueChanged;
    }
}
void TWPropertiesPrivate::_q_on_SelectionChanged()
{
    TSelectionShapeSP Select = INSTANCE(ShapeManager)->Selection();
    if (Select->count() > 0)
        Restore_TVtData_MetaData(Select->SelectedShapes().front());
}
void TWPropertiesPrivate::_q_on_VtSlider_valueChanged(double V)
{
    Q_UNUSED(V);
    if (IgnoreValueChanged) return;
    out::TVtData Vt1;
    GetVtDara(Vt1);
    TShapeList Selected = INSTANCE(ShapeManager)->Selection()->SelectedShapes();

    typedef TDataShape<out::TVtData> TVtType;
    TVtType *VtType;
    foreach (TShapeSP Shape, Selected)
    {
        if (Shape->objectName() == TImageShape::ShapeName() && Selected.size() > 1)  continue;
        VtType = dynamic_cast<TVtType *>(Shape.data());
        if (VtType) {
            VtType->MetaData1() = Vt1;
        }
    }
}
void TWPropertiesPrivate::_q_on_LEdSSpeed_valueChanged(double S)
{
    QSettings settings("Param/lithography.conf", QSettings::IniFormat);
    settings.setValue("ScanSpeed", S);

    if (IgnoreValueChanged) return;
    TShapeList Selected = INSTANCE(ShapeManager)->Selection()->SelectedShapes();

    typedef TDataShape<out::TVtData> TVtType;
    TVtType *VtType;
    foreach (TShapeSP Shape, Selected) {
        VtType = dynamic_cast<TVtType *>(Shape.data());
        if (VtType) VtType->Speed() = S;
    }
}


//=====================================================================================================================

TWProperties::TWProperties(QWidget *parent) : QWidget(parent), d_ptr(new TWPropertiesPrivate)
{
    Q_D(TWProperties);
    d->q_ptr = this;
    d->Construct();
}
DoubleChanger *TWProperties::getSpeedSlider()
{
    Q_D(TWProperties);
    return d->DCsS;
}

void TWProperties::setForceLithName()
{
    Q_D(TWProperties);
    d->GrBParameters1->setTitle(tr("Pulse SetPoint"));
    d->DCsV->show();
    d->DCsT->hide();
}

void TWProperties::setVoltLithName()
{
    Q_D(TWProperties);
    d->GrBParameters1->setTitle(tr("Pulse Voltage, time"));
    d->DCsV->show();
    d->DCsT->show();
}
void TWProperties::setLaserMode()
{
    Q_D(TWProperties);
    d->GrBParameters1->setTitle(tr("Pulse time"));
    d->DCsV->hide();
    d->DCsT->show();
}

}

#include "moc_TWProperties.cpp"
