#include "TConversion.h"
#include "TConversion_p.h"
#include "shape/ShapeManager.h"

//#include <QDebug>
namespace graphed {

TConversionPrivate::TConversionPrivate()  { }

void TConversionPrivate::InvalidateCache()
   {
      GlobalBoundingRect.ValueIsSet = false;
   }
QRectF TConversionPrivate::UpdateGlobalBoundingRect() const
   {
      Q_Q(const TConversion);
      GlobalBoundingRect = ViewRect;
      if (INSTANCE(ShapeManager)->count()) GlobalBoundingRect |= q->DocumentToView(INSTANCE(ShapeManager)->BoundingRect());
      GlobalBoundingRect.Var.adjust(0, 0, 2. * Margin, 2. * Margin);
      return GlobalBoundingRect;
   }


//======================================================================================================================

TConversion::TConversion(): TConversionBase(*new TConversionPrivate)
   {
      Q_D(TConversion);
      d->q_ptr = this;

      connect(INSTANCE(ShapeManager), SIGNAL(ShapeChanged(TShapeSP)), this, SLOT(on_ShapeChanged(TShapeSP)));
   }

void TConversion::ZoomBy(const QPointF &Center, const qreal zoom)
   {
      Q_D(TConversion);
      //qDebug() << d->ViewRect << endl;
      const QPointF CenterInVRCoord = Center;
      //const QPointF CenterInVRCoord = (Center - d->ViewRect.topLeft()) / Zoom();//WidgetToViewRect(Center);
      //qDebug() << d->ViewRect << Center << CenterInVRCoord;
      d->ViewRect.setTop   (CenterInVRCoord.y() - zoom * (CenterInVRCoord.y() - d->ViewRect.top()));
      d->ViewRect.setBottom(CenterInVRCoord.y() + zoom * (d->ViewRect.bottom() - CenterInVRCoord.y()));
      d->ViewRect.setLeft  (CenterInVRCoord.x() - zoom * (CenterInVRCoord.x() - d->ViewRect.left()));
      d->ViewRect.setRight (CenterInVRCoord.x() + zoom * (d->ViewRect.right() - CenterInVRCoord.x()));
      //qDebug() << d->ViewRect << endl;

      SetZoom(Zoom() * zoom);
      SetOffset(d->ViewRect.topLeft());
      d->UpdateGlobalBoundingRect();
   }
QPointF TConversion::WidgetToViewRect(const QPointF &Pos) const
   {
      return Pos - GlobalBoundingRect().topLeft();
   }
void TConversion::on_ShapeChanged(TShapeSP Shape)
   {
      Q_UNUSED(Shape);
      Q_D(TConversion);
      QRectF Old = d->GlobalBoundingRect;
      if (Old != d->UpdateGlobalBoundingRect()) emit GlobalBoundingRectChanged(d->GlobalBoundingRect);
   }
void TConversion::InvalidateCache()
   {
      Q_D(TConversion);
      d->GlobalBoundingRect.ValueIsSet = false;
   }
QRectF TConversion::GlobalBoundingRect() const
   {
      Q_D(const TConversion);
      if (d->GlobalBoundingRect.IsValueSet()) return d->GlobalBoundingRect;
      else return d->UpdateGlobalBoundingRect();
   }
void TConversion::DocumentResolutionChanged()
   {
      //InvalidateCache();
   }
void TConversion::OffsetChanged()
   {
      InvalidateCache();
   }
void TConversion::ViewRectChanged()
   {
      InvalidateCache();
   }
void TConversion::VirginViewRectChanged()
   {
      //InvalidateCache();
   }

}
