#include "TConversionBase.h"
#include "TConversionBase_p.h"
#include <QRectF>
#include <QDebug>

namespace graphed {

TConversionBasePrivate::TConversionBasePrivate(): Margin(10)
   {
      Zoom = 1;
   }

//======================================================================================================================

TConversionBase::TConversionBase(TConversionBasePrivate &dd): d_ptr(&dd) { }
TConversionBase::TConversionBase(): d_ptr(new TConversionBasePrivate)
   {
      Q_D(TConversionBase);
      d->q_ptr = this;
   }
TConversionBase::~TConversionBase() { delete d_ptr; }
QPointF TConversionBase::Offset() const
   {
      Q_D(const TConversionBase);
      return d->Offset;
   }
void TConversionBase::SetOffset(const QPointF &o)
   {
      Q_D(TConversionBase);
      d->Offset = o;
      d->ViewRect.moveTopLeft(o);
      OffsetChanged();
   }
qreal TConversionBase::Zoom() const
   {
      Q_D(const TConversionBase);
      return d->Zoom;
   }
void TConversionBase::SetZoom(const qreal z)
   {
      Q_D(TConversionBase);
      d->Zoom = z;
      ZoomChanged();
   }
QSize TConversionBase::DocumentResolution() const
   {
      Q_D(const TConversionBase);
      return d->DocumentResolution;
   }
void TConversionBase::SetDocumentResolution(const QSize &s)
   {
      Q_D(TConversionBase);
      if (d->DocumentResolution == s) return;
      if (s.isEmpty()) qDebug() << "TConversionBase::SetDocumentResolution: s is empty!";
      d->DocumentResolution = s;
      DocumentResolutionChanged();
   }
QRectF TConversionBase::DocumentRect() const
   {
      Q_D(const TConversionBase);
      return d->DocumentRect;
   }
void TConversionBase::SetDocumentRect(const QRectF &Rect)
   {
      Q_D(TConversionBase);
      if (d->DocumentRect == Rect) return;
      if (Rect.isEmpty()) qDebug() << "TConversionBase::SetDocumentRect: Rect is empty!";
      d->DocumentRect = Rect;
      DocumentRectChanged();
   }
QPointF TConversionBase::WidgetToDocument(const QPointF &WidPos) const
   {
      return (WidPos - Offset()) / Zoom();
   }
QPointF TConversionBase::DocumentToView(const QPointF &DocPos) const
   {
      return DocPos * Zoom() + Offset();
   }
QRectF TConversionBase::DocumentToView(const QRectF &DocRect) const
   {
      return QRectF(DocumentToView(DocRect.topLeft()), DocumentToView(DocRect.bottomRight()));
   }
QPointF TConversionBase::DocumentToWidget(const QPointF &DocPos) const
   {
      return (DocPos + Offset()) * Zoom();
   }
qreal TConversionBase::Margin() const
   {
      Q_D(const TConversionBase);
      return d->Margin;
   }
QPointF TConversionBase::ViewToPhysical(const QPointF &V) const
   {
      Q_D(const TConversionBase);
      if (d->DocumentRect.isEmpty()) return V;
      QRectF rect = d->DocumentRect;
      rect = QRectF(rect.left(), rect.bottom(), rect.width(), -rect.height());
      return GeneralMap(d->ViewRect, V, rect);
      /*const qreal XRelVal = V.x() / d->ViewRect.width(), YRelVal = V.y() / d->ViewRect.height();
      return QPointF(d->DocumentRect.left() + XRelVal * d->DocumentRect.width(), d->DocumentRect.top() + YRelVal * d->DocumentRect.height());*/
   }
QPointF TConversionBase::DocumentToPhysical(const QPointF &D) const
   {
      return ViewToPhysical(D * Zoom());
   }
/*void TConversionBase::DocumentToPhysical(STLPointFVector &V) const
   {
      Q_D(const TConversionBase);
      if (d->DocumentRect.isEmpty()) return;

      for (STLPointFVector::iterator It = V.begin(); It != V.end(); It++)
         *It = DocumentToPhysical(*It);
   }*/
QPointF TConversionBase::PhysicalTo(const QRectF &ToIt, const QPointF &PhysP) const
   {
      Q_D(const TConversionBase);
      if (d->DocumentRect.isEmpty()) return PhysP;
      QRectF rect = d->DocumentRect;
      rect = QRectF(rect.left(), rect.bottom(), rect.width(), -rect.height());
      return GeneralMap(rect, PhysP, ToIt);
      /*const qreal XRelVal = PhysP.x() / d->DocumentRect.width(), YRelVal = PhysP.y() / d->DocumentRect.height();
      return QPointF(ToIt.left() + XRelVal * ToIt.width(), ToIt.top() + YRelVal * ToIt.height());*/
   }
QPointF TConversionBase::GeneralMap(const QRectF &FromR, const QPointF &FromP, const QRectF &ToR)
   {
      //if (FromR.isEmpty()) qDebug() << "TConversionBase::GeneralMap: FromR is empty!!!";
      const qreal XRelVal = (FromP.x()-FromR.left()) / FromR.width(), YRelVal = (FromP.y()-FromR.top()) / FromR.height();
      return QPointF(ToR.left() + XRelVal * ToR.width(), ToR.top() + YRelVal * ToR.height());
   }
QPoint TConversionBase::VirginVRToResolution(const QPointF &VVR) const
   {
      Q_D(const TConversionBase);
      if (d->DocumentResolution.isEmpty()) return VVR.toPoint();
      QPointF Res = GeneralMap(d->VirginViewRect, VVR, QRectF(QPointF(), d->DocumentResolution));
      return QPoint(static_cast<int>(Res.x()), static_cast<int>(Res.y()));
      //return Res.toPoint();
   }
QPointF TConversionBase::VirginVRToResolutionF(const QPointF &VVR) const
   {
      Q_D(const TConversionBase);
      if (d->DocumentResolution.isEmpty()) return VVR;
      return GeneralMap(d->VirginViewRect, VVR, QRectF(QPointF(), d->DocumentResolution));
   }
QRectF TConversionBase::VirginVRToResolutionF(const QRectF &VVR) const
   {
      return QRectF(VirginVRToResolutionF(VVR.topLeft()), VirginVRToResolutionF(VVR.bottomRight()));
   }
QPointF TConversionBase::ResolutionToPhysical(const QPointF R) const
   {
      Q_D(const TConversionBase);
      if (d->DocumentResolution.isEmpty()) return R;
      QRectF rect = d->DocumentRect;
      rect = QRectF(rect.left(), rect.bottom(), rect.width(), -rect.height());
      return GeneralMap(QRectF(QPointF(), d->DocumentResolution), R, rect);
   }
void TConversionBase::SetViewRect(const QRectF &Rect)
   {
      Q_D(TConversionBase);
      if (d->ViewRect == Rect) return;
      if (Rect.isEmpty()) qDebug() << "TConversionBase::SetViewRect: Rect is empty!";
      d->VirginViewRect = d->ViewRect = Rect;
      ViewRectChanged(); VirginViewRectChanged();
      d->Offset = Rect.topLeft();
      OffsetChanged();
   }
QRectF TConversionBase::ViewRect() const
   {
      Q_D(const TConversionBase);
      return d->ViewRect;
   }
QRectF TConversionBase::VirginViewRect() const
   {
      Q_D(const TConversionBase);
      return d->VirginViewRect;
   }


}
