#ifndef TCONVERSIONBASE_H
#define TCONVERSIONBASE_H

#include <QObject>
#include "core/core.h"

class QPointF;
class QSize;
class QRectF;

namespace graphed {

class TConversionBase;
class TConversionBasePrivate;

class TConversionBase: public QObject
   {
      QTPIMPL_PRIVATE(TConversionBase)
   protected:
      TConversionBase(TConversionBasePrivate &dd);

      virtual void ZoomChanged()          { }
      virtual void OffsetChanged()         { }
      virtual void DocumentRectChanged()    { }
      virtual void ViewRectChanged()         { }
      virtual void VirginViewRectChanged()    { }
      virtual void DocumentResolutionChanged() { }
   public:
      explicit TConversionBase();
      virtual ~TConversionBase();

      QPointF Offset() const;
      void SetOffset(const QPointF &o);

      qreal Zoom() const;
      void SetZoom(const qreal z);

      QSize DocumentResolution() const;
      void SetDocumentResolution(const QSize &s);
      QRectF DocumentRect() const;
      void SetDocumentRect(const QRectF &Rect);
      QRectF ViewRect() const;
      void SetViewRect(const QRectF &Rect);
      QRectF VirginViewRect() const;

      static QPointF GeneralMap(const QRectF &FromR, const QPointF &FromP, const QRectF &ToR);
      QPointF DocumentToWidget(const QPointF &DocPos) const;
      QPointF WidgetToDocument(const QPointF &WidPos) const;
      QPointF DocumentToView(const QPointF &DocPos) const;
      QRectF DocumentToView(const QRectF &DocRect) const;
      QPointF ViewToPhysical(const QPointF &V) const;
      QPointF DocumentToPhysical(const QPointF &D) const;
      QPointF PhysicalTo(const QRectF &ToIt, const QPointF &PhysP) const;
      QPoint VirginVRToResolution(const QPointF &VVR) const;
      QPointF VirginVRToResolutionF(const QPointF &VVR) const;
      QRectF VirginVRToResolutionF(const QRectF &VVR) const;
      QPointF ResolutionToPhysical(const QPointF R) const;
      //void DocumentToPhysical(STLPointFVector &V) const;

      qreal Margin() const;
   };

}

#endif // TCONVERSIONBASE_H
